import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import {
  AggregateID,
  ConflictException,
  UniqueConflictException,
} from '@mobicoop/ddd-library';
import { CreateAuthenticationCommand } from './create-authentication.command';
import { AUTHENTICATION_REPOSITORY } from '@modules/newauthentication/authentication.di-tokens';
import { AuthenticationRepositoryPort } from '../ports/authentication.repository.port';
import { AuthenticationEntity } from '@modules/newauthentication/core/domain/authentication.entity';
import {
  AuthenticationAlreadyExistsException,
  UsernameAlreadyExistsException,
} from '@modules/newauthentication/core/domain/authentication.errors';

@CommandHandler(CreateAuthenticationCommand)
export class CreateAuthenticationService implements ICommandHandler {
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly repository: AuthenticationRepositoryPort,
  ) {}

  async execute(command: CreateAuthenticationCommand): Promise<AggregateID> {
    const authentication = await AuthenticationEntity.create({
      password: command.password,
      usernames: command.usernames,
    });

    try {
      await this.repository.insert(authentication);
      return authentication.id;
    } catch (error: any) {
      console.log('error', error.cause);
      if (error instanceof ConflictException) {
        throw new AuthenticationAlreadyExistsException(error);
      }
      if (
        error instanceof UniqueConflictException &&
        error.message.includes('username')
      ) {
        throw new UsernameAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
